/*
  File: basicdll.c

  This is the C source code for a bare-bones DLL written in C.  It
  consists of two functions:

    DllEntryPoint - This function is called by Windows when
                    basicdll.dll is loaded (and when other
                    processes attach to the DLL to use its
                    functions).

    Exported Function - This function is here just to demonstrate
                        what an exported function looks like.  Looks
                        like a pretty ordinary function, huh?  It
                        calls the Windows API function MessageBox()
                        just to let you know that the function was
                        called.

*/

/* windows.h holds all the definitions of the
   various functions, structures, and constants
   the WindowsAPI uses. */
#include <windows.h>

/* basicdll.h contains the foreward declarations
   for the two functions we define in this source
   file */
#include "basicdll.h"

/*
  DllEntryPoint

  Preconditions: none - it is called at various times during the DLL's
                 life (when first loaded, when processes attach/detach,
                 etc).

  PostConditions: none - this implementation of DllEntryPoint doesn't
                  really do anything.  Normally, this function would
                  look at the fdwReason parameter and allocate/free
                  memory depending on what fdwReason is.

  Parameters:
    HINSTANCE hDLL:  The HINSTANCE of this dll.  You save this variable
                     and pass it to Windows API functions that require
                     an HINSTANCE parameter.

    DWORD fdwReason: This is a numeric code that tells us why Windows
                     is calling DllEntryPoint()

    LPVOID lpvReserved: This parameter is not used - it is reserved
                        by Windows for a possible future use.  Microsoft
                        likes to do this stuff sometimes...

  Return value:  Returns TRUE if initialization completes succesfully,
                 It returns FALSE if something goes wrong and the DLL
                 is unable to initialize.

*/
BOOL WINAPI DllEntryPoint( HINSTANCE hDll,
                           DWORD     fdwReason,
                           LPVOID    lpvReserved )
{
  /*
    Let's find out exactly why Windows called us...
  */
  switch( fdwReason )
  {
    case DLL_PROCESS_ATTACH:
      /*
        Here we would normally do some startup stuff, like saving the
        hDll parameter to a global variable, allocating memory for
        local structures and stuff, etc...
      */
      break;

    case DLL_PROCESS_DETACH:
      /*
        Here we would free the memory and stuff we allocated in
        response to DLL_PROCESS_ATTACH.
      */
      break;

    default:
      /*
        If fdwReason isn't the above two cases, we'll just ignore it
      */
      break;
  };

  /*
    If we had run into an error (like not being able to allocate memory
    or something else) we would return FALSE.  This would tell Windows
    that we were unnable to succesfully initialize.  If this DLL was
    being loaded by a program as it started, the program would fail to
    load.  If this DLL was being loaded by a running program via a call
    to LoadLibrary(), LoadLibrary would return NULL to the calling program.
  */
  return TRUE;
}

/*
  ExportedFunction

  Preconditions: none
  Postcontitions: none

  Parameters:
    none

*/
int WINAPI ExportedFunction( void )
{
  /*
    Let's call MessageBox() to display a nice friendly notification
    that we were called.  Another program or DLL can call this function
    by either linking to this DLL at compile time, or calling
    LoadLibrary() and GetProcAddress().
  */

  MessageBox( NULL,  //HWINDOW of the window that owns the message box
              "Hello from inside basicdll.dll!", //Title
              "ExportedFunction() has been called!",//Text
              MB_OK | MB_ICONINFORMATION );//Bit mask flags

  return 1;
}

